﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Drawing;
using System.Windows.Forms;
using GradientBoostedTrees;
using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Algorithms.DataAnalysis.Views {
  [View("Regression tree model")]
  [Content(typeof(RegressionTreeModel), true)]
  public partial class RegressionTreeModelView : AsynchronousContentView {
    public new RegressionTreeModel Content {
      get { return (RegressionTreeModel)base.Content; }
      set {
        base.Content = value;
      }
    }

    public RegressionTreeModelView()
      : base() {
      InitializeComponent();
      symbolicExpressionTreeChart.TextFont = new System.Drawing.Font(FontFamily.GenericSerif, 8F);
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        // clear
        symbolicExpressionTreeChart.Tree = null;
      } else {
        symbolicExpressionTreeChart.Tree = CreateSymbTree(Content);
      }
    }

    private ISymbolicExpressionTree CreateSymbTree(RegressionTreeModel regressionTreeModel) {
      int treeNodeIdx = 0;
      return new SymbolicExpressionTree(CreateSymbTree(regressionTreeModel.tree, ref treeNodeIdx));
    }

    private ISymbolicExpressionTreeNode CreateSymbTree(RegressionTreeModel.TreeNode[] tree, ref int treeNodeIdx) {
      var node = tree[treeNodeIdx];
      treeNodeIdx++;

      if (node.varName != RegressionTreeModel.TreeNode.NO_VARIABLE) {
        var treeNode = (DecisionTreeNode)decisionSy.CreateTreeNode();
        treeNode.VariableName = node.varName;
        treeNode.Threshold = node.val;
        treeNode.AddSubtree(CreateSymbTree(tree, ref treeNodeIdx));
        treeNode.AddSubtree(CreateSymbTree(tree, ref treeNodeIdx));
        return treeNode;
      } else {
        var treeNode = (ConstantTreeNode)constantSy.CreateTreeNode();
        treeNode.Value = node.val;
        return treeNode;
      }
    }

    #region helper types for symbols
    private static readonly Decision decisionSy = new Decision();
    private static readonly Constant constantSy = new Constant();


    // TODO use simple symbols
    [NonDiscoverableType]
    public sealed class Decision : Symbol {
      private const int minimumArity = 2;
      private const int maximumArity = 2;

      public override int MinimumArity {
        get { return minimumArity; }
      }
      public override int MaximumArity {
        get { return maximumArity; }
      }

      public Decision() : base("Decision", "") { }
      public override IDeepCloneable Clone(Cloner cloner) {
        throw new System.NotImplementedException();
      }
      public override ISymbolicExpressionTreeNode CreateTreeNode() {
        return new DecisionTreeNode(this);
      }
    }
    [NonDiscoverableType]
    public sealed class DecisionTreeNode : SymbolicExpressionTreeNode {
      public new Decision Symbol {
        get { return (Decision)base.Symbol; }
      }

      public double Threshold { get; set; }
      public string VariableName { get; set; }

      public DecisionTreeNode(Decision symbol) : base(symbol) { }

      public override bool HasLocalParameters {
        get {
          return true;
        }
      }
      public override IDeepCloneable Clone(Cloner cloner) {
        throw new System.NotImplementedException();
      }

      public override string ToString() {
        return string.Format("{0} <= {1:E4}", VariableName, Threshold);
      }
    }

    [NonDiscoverableType]
    public sealed class Constant : Symbol {
      private const int minimumArity = 0;
      private const int maximumArity = 0;

      public override int MinimumArity {
        get { return minimumArity; }
      }
      public override int MaximumArity {
        get { return maximumArity; }
      }

      public Constant() : base("Constant", "") { }
      public override IDeepCloneable Clone(Cloner cloner) {
        throw new System.NotImplementedException();
      }
      public override ISymbolicExpressionTreeNode CreateTreeNode() {
        return new ConstantTreeNode(this);
      }
    }

    [NonDiscoverableType]
    public sealed class ConstantTreeNode : SymbolicExpressionTreeTerminalNode {
      public new Constant Symbol {
        get { return (Constant)base.Symbol; }
      }

      private double constantValue;
      public double Value {
        get { return constantValue; }
        set { constantValue = value; }
      }

      private ConstantTreeNode() : base() { }
      public ConstantTreeNode(Constant constantSymbol) : base(constantSymbol) { }

      public override bool HasLocalParameters {
        get {
          return true;
        }
      }
      public override IDeepCloneable Clone(Cloner cloner) {
        throw new System.NotImplementedException();
      }

      public override string ToString() {
        return constantValue.ToString("E4");
      }
    }

    #endregion
  }
}
